#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
 
double rk4(double(*f)(double, double, double, double), double dx, double x, double y, double r, double K)
{
	double	k1 = dx * f(x, y, r, K),
			k2 = dx * f(x + dx / 2, y + k1 / 2, r, K),
			k3 = dx * f(x + dx / 2, y + k2 / 2, r, K),
			k4 = dx * f(x + dx, y + k3, r, K);
	return y + (k1 + 2 * k2 + 2 * k3 + k4) / 6;
}
 
double rate(double x, double y, double r, double K)
{
	return r*y*((K-y)/K);
}
 
int main(void)
{
	double *y, x, y2, h, y0,r, K;
	double x0, x1;
	int i, n, tic, toc;
	FILE * gnuplotPipe = fopen ("output.dat", "w");
 	
 	//enter argument
	printf("Enter the starting population: ");
	scanf("%lf", &y0);
	//enter argument
	printf("Enter r: ");
	scanf("%lf", &r);
	//enter argument
	printf("Enter K: ");
	scanf("%lf", &K);
	//enter argument
	printf("Enter the start time: ");
	scanf("%lf", &x0);
	//enter argument
	printf("Enter the end time: ");
	scanf("%lf", &x1);
	//enter argument
	printf("Enter the step size: ");
	scanf("%lf", &h);

	n = 1 + (x1 - x0)/h;

	y = malloc(sizeof(double) * n);
	y[0] = y0;

 	tic = clock();
	for (i = 1; i < n; i++)
		y[i] = rk4(rate, h, x0 + h * (i - 1), y[i-1], r, K);
	toc = clock();

	// Measure elapsed time
	double elapsedtime = (double)(toc - tic) / CLOCKS_PER_SEC;

	printf("x\ty\n----------------\n");
	for (i = 0; i <= x1; i+=h) {
		x = x0 + h * i;
		y2 = pow(x * x / 4 + 1, 2);
		printf("%g\t%g\n", x, y[i]);
		fprintf(gnuplotPipe, "%f, %f\n", x, y[i]);
	}
	fclose(gnuplotPipe);

	printf("\nThe program has computed %d steps\n", n);
	printf("It was executed in %g secondes\n", elapsedtime);
	printf ("The program has computed %.3g steps per second\n", n/elapsedtime);
 
	return 0;
}